@@ -35,6 +35,7 @@
 #include "ni_globals.h"
 #include <NetInfo/system_log.h>
 #include <NetInfo/mm.h>
+#include <NetInfo/dsstore.h>
 #include "index.h"
 #include "index_manager.h"
 
@@ -292,6 +293,13 @@ ni_init(char *rootdir, void **handle)
 	}
 
 	ni->im_hdl = im_alloc();
+
+	/*
+	 * Make sure the index manager cache is flushed
+	 * when the store is changed by another process.
+	 */
+	dsstore_set_sync_delegate((dsstore *)ni->file_hdl, (void (*)(void *))im_forget, &ni->im_hdl);
+
 	*handle = ni;
 
 	return status;
@@ -487,6 +495,8 @@ ni_create(void *handle, ni_id *parent_id, ni_proplist pl, ni_id *child_id_p, ni_
 	obj_free(child);
 	obj_free(parent);
 
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
@@ -556,6 +566,8 @@ ni_destroy(void *handle, ni_id *parent_id, ni_id child_id)
 		obj_unalloc(handle, child);
 	}
 
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
@@ -622,6 +634,9 @@ ni_write(void *handle, ni_id *id, ni_proplist props)
 
 	*id = obj->nio_id;
 	obj_free(obj);
+
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
@@ -1024,6 +1039,8 @@ ni_writeprop(void *handle, ni_id *id, ni_index prop_index, ni_namelist values)
 	*id = obj->nio_id;
 	obj_free(obj);
 
+	file_notify(FH(handle));
+
 	return status;
 }
 
@@ -1059,6 +1076,8 @@ ni_createprop(void *handle, ni_id *id, ni_property prop, ni_index where)
 	im_create_list(IMH(handle), obj, prop.nip_name, prop.nip_val);
 	obj_free(obj);
 
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
@@ -1106,6 +1125,8 @@ ni_destroyprop(void *handle, ni_id *id, ni_index prop_index)
 	ni_prop_free(&saveprop);
 	obj_free(obj);
 
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
@@ -1150,6 +1171,8 @@ ni_renameprop(void *handle, ni_id *id, ni_index prop_index, ni_name_const name)
 	*id = obj->nio_id;
 	obj_free(obj);
 
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
@@ -1219,6 +1242,8 @@ ni_createname(void *handle, ni_id *id, ni_index prop_index, ni_name_const name,
 	*id = obj->nio_id;
 	obj_free(obj);
 
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
@@ -1339,6 +1364,8 @@ ni_writename(void *handle, ni_id *id, ni_index prop_index, ni_index name_index,
 	*id = obj->nio_id;
 	obj_free(obj);
 
+	file_notify(FH(handle));
+
 	return NI_OK;
 }
 
